%-------------------------------------------------------------------------%
% VERIFICATION OF AN AGENT-BASED DISEASE MODEL OF HUMAN MYCOBACTERIUM TUBERCULOSIS INFECTION
% Cristina Curreli*, Francesco Pappalardo, Giulia Russo, Marzio Pennisi, Dimitrios Kiagias, Miguel Juarez, Marco Viceconti
%
% Time Step convergence analsysis
% Inputs: 
% - simulation results (file.out) obtained with different TS size saved in
%   different folders
% Outputs: 
% - raw data related to the OVs (file.csv) with different TS
% - summary of the 12 output quantities with different TS, Mo: three key features (PV, TPV
%   and FV), for each one of the four output variables of interest Th17, Ab,
%   B and TotAGS (_Summary.csv file)
% - summary of the percentage discretization error according to Eq.1 of the
%   paper (_Summary_e.csv file)
%
%    Author: Cristina Curreli, 2020
%    email: *cristina.curreli@unibo.it
%-------------------------------------------------------------------------%
%% Script Inputs

clear all
close all
clc

%select the folder where you saved the simulation results related to the
%time step convergence analysis
pathfile = uigetdir ();

%create new folder with results
FolderResults = "_Results";
mkdir _Results
PathResults = fullfile(pathfile,FolderResults);

%names of the folders where the simulation results are saved - make sure
%name "01" is related to the largest TS and the last number (in this case
%"14" is related to the smallest TS
Folders = ["01","02","03","04","05","06","07","08","09","10","11","12","13","14"]

%names of the simulation result file where the OVs are saved
namefileTH = 'TH-data.out';
namefileAB = 'Ab-data.out';
namefileB = 'B-data.out';
namefileAGS = 'Ag-data.out';

% number of columns of the Summary matrix results (TimeStep in seconds;
% n_iterations; M0 (3x4 =12 -> PV, TPV and FV for each one of the four
% output variables of interest Th17, Ab, B and TotAGS)
n_col = 14;
Summary = zeros (length(Folders),n_col); % matrix for Mo results
Summary_e = zeros ((length(Folders)-1),n_col); % matrix for percentage discretization error results

%% Read the OVs + key features of the Ovs concentration trends; then save raw data in a _Results folder

for i=1:length(Folders)

    Folder = Folders(i)
    Path = fullfile(pathfile,Folder)
    
    % READ TH-data.out AND SELECT/EXPORT INFO THAct17 COLUMN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    filenameTH = fullfile(Path,namefileTH);
    TH = importdata(filenameTH);
    THdata = TH.data;
    % Convert to output type
    Time = THdata(:,1);
    ThAct17 = THdata(:,10);
    PV_ThAct17 = max(ThAct17);
    i_TimePV_ThAct17 = find(ThAct17==PV_ThAct17);
    TimePV_ThAct17 = Time(i_TimePV_ThAct17);
    FV_ThAct17 = ThAct17(end);
    n_iteraz = length (Time);
    Time_step = Time(2)

    % READ Ab-data.out AND SELECT/EXPORT INFO totIg COLUMN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    filenameAB = fullfile(Path,namefileAB);
    Ab = importdata(filenameAB);
    Abdata = Ab.data;
    % Convert to output type
    Abtot = Abdata(:,6);
    PV_Ab = max(Abtot);
    i_TimePV_Ab = find(Abtot==PV_Ab);
    TimePV_Ab = Time(i_TimePV_Ab);
    FV_Ab = Abtot(end);

    % READ B-data.out AND SELECT/EXPORT INFO sum(BMem, BNotMem, BAct, BpresII, BDuplic and Bresting)  [sum of columns 3-6 of the B-data.out] %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    filenameB = fullfile(Path,namefileB);
    B = importdata(filenameB);
    Bdata = B.data;
    % Convert to output type
    BMem = Bdata(:,3);
    BNotMem = Bdata(:,4);
    BAct = Bdata(:,5);
    BpresII = Bdata(:,6);
    BDuplic = Bdata(:,7);
    Bresting = Bdata(:,8);
    Btot = BAct;
    PV_Btot = max(Btot);
    i_TimePV_Btot = find(Btot==PV_Btot);
    TimePV_Btot_all = Time(i_TimePV_Btot);
    TimePV_Btot = TimePV_Btot_all(1);
    FV_Btot = Btot(end);

    % READ Ag-data.out AND SELECT/EXPORT INFO tot Ags %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
    filenameAGS = fullfile(Path,namefileAGS);
    AGS = importdata(filenameAGS);
    AGSdata = AGS.data;
    indiceAGS2max = 5000000/AGS.data(2,1); %5000000 = t after which we always have the second peack.. check visually before selecting  
    % Convert to output type
    AGStot = AGSdata(:,3);
    AGStotNew = AGSdata(indiceAGS2max:end,3);
    TimeNew = AGSdata(indiceAGS2max:end,1);
    PV_AGStot = max(AGStotNew);
    i_TimePV_AGS = find(AGStotNew==PV_AGStot);
    TimePV_AGStot = TimeNew(i_TimePV_AGS(1));
    FV_AGStot = AGStot(end);

    % Write file with summary of the OVs raw data
    [~,basename,~]=fileparts(Path);
    table_results_matrix = [Time,ThAct17,Abtot,Btot,AGStot];
    table_results = array2table (table_results_matrix);
    table_results.Properties.VariableNames = {'Time', 'Th17', 'Ab', 'B', 'TotAGS'};
    writetable(table_results,fullfile(PathResults, strcat(basename, '_OVs_RawData.csv')),'WriteVariableNames',true);
    table_sum_matrix = [Time_step,n_iteraz,PV_ThAct17,PV_Ab,PV_Btot,PV_AGStot,....
        TimePV_ThAct17,TimePV_Ab,TimePV_Btot,TimePV_AGStot,FV_ThAct17,FV_Ab,FV_Btot,FV_AGStot];
    Summary(i,:) = table_sum_matrix;

end

%% Save Summary Results in a _Summary.csv and _QSummary.csv

table_summary = array2table (Summary);
table_summary.Properties.VariableNames = {'TS','num_iteraz','PV_ThAct17','PV_Ab','PV_Btot','PV_AGStot',....
    'TimePV_ThAct17','TimePV_Ab','TimePV_Btot','TimePV_AGStot','FV_ThAct17','FV_Ab','FV_Btot','FV_AGStot'};
writetable(table_summary,fullfile(PathResults, '_Summary.csv'),'WriteVariableNames',true);

Summary_e=  Summary(1:leng,:)
leng=length(Folders)-1;
for j=3:6
    for ii=1:leng
        e = (abs((Summary(leng+1,j)-(Summary(ii,j)))/Summary(leng+1,j)))*100;
        Summary_e(ii,j)=e;
    end
end
for j=7:10
    for ii=1:leng
        e = (abs((Summary(leng+1,j)-(Summary(ii,j)))/Summary(leng+1,j)))*100;
        Summary_e(ii,j)=e;
    end
end

Average_ThAct17 = mean(ThAct17);
Average_Abtot = mean(Abtot);
Average_Btot = mean(Btot);
Average_AGStot = mean(AGStot);
AverageOVs = [Average_ThAct17 Average_Abtot Average_Btot Average_AGStot];
for j=11:length(Folders)
    for ii=1:leng
        e = (abs((Summary(leng+1,j)-(Summary(ii,j)))/AverageOVs(j-10)))*100;
        Summary_e(ii,j)=e;
    end
end
    
table_summary_e = array2table (Summary_e);
table_summary_e.Properties.VariableNames = {'TS','num_iteraz','e_PV_ThAct17','e_PV_Ab','e_PV_Btot','e_PV_AGStot',....
    'e_TimePV_ThAct17','e_TimePV_Ab','e_TimePV_Btot','e_TimePV_AGStot','e_FV_ThAct17','e_FV_Ab','e_FV_Btot','e_FV_AGStot'};
writetable(table_summary_e,fullfile(PathResults, '_Summary_e.csv'),'WriteVariableNames',true);       
